@@ -17,6 +17,15 @@ import os
 import re
 import logging
 
+try:
+    from functools import lru_cache
+except ImportError:  # Python2
+
+    def lru_cache():
+        "dummy no op alternative for Python 2"
+        return lambda f: f
+
+
 # FIXME: when we drop Python 2.4, this should become 'from . import path'
 from LbEnv.ProjectEnv import path, Error
 from LbEnv.ProjectEnv.version import DEFAULT_VERSION, versionKey as _vkey, LCGInfoName
@@ -57,7 +66,7 @@ def platform_sort_key(platform):
     else:
         arch, os_id, comp, opt = platform.split("-")
     os_id = tuple(int(i) for i in os_id if i.isdigit())
-    comp = tuple(int(i) for i in comp if i.isdigit())
+    comp = compilerKey(comp)
     return ("0" if opt == "do0" else opt, arch, comp, os_id)
 
 
@@ -79,6 +88,23 @@ def versionKey(x):
     return _vkey(x[0])
 
 
+def compilerKey(comp):
+    family, version, extension = re.match(
+        r"^([^\d]+)(\d+)+(\+.+)?$", comp, re.IGNORECASE
+    ).groups()
+    # Between gcc 3.4 and gcc 6.2, two digits were used for the version
+    # Append "0" to all other versions to make the subsequent comparison work
+    if not (3 <= int(version[0]) <= 6):
+        version += "0"
+    version = int(version)
+    try:
+        family_weight = ["icc", "clang", "gcc"].index(family)
+    except ValueError:
+        family_weight = -1  # unknown family
+    # give highest weight to no extension otherwise sort lexical order
+    return family_weight, version, not extension, extension
+
+
 class NotFoundError(Error):
     """
     Generic error for configuration elements that are not found.
@@ -152,6 +178,7 @@ def findFile(name, search_path=None):
     return fn
 
 
+@lru_cache()
 def findProject(name, version, platform, allow_empty_version=False):
     """
     Find a Gaudi-based project in the directories specified in the 'path'
@@ -169,13 +196,24 @@ def findProject(name, version, platform, allow_empty_version=False):
     project.
     """
     log.debug(
-        "findProject(name=%r, version=%r, platform=%r, " "allow_empty_version=%r)",
+        "findProject(name=%r, version=%r, platform=%r, allow_empty_version=%r)",
         name,
         version,
         platform,
         allow_empty_version,
     )
 
+    if re.match(r"^\d+\.\d+(\.\d+)*$", version):
+        # If we are requested to find a X.Y[.Z] version (new style CMake config)
+        # we have to look for directories with vXrY[pZ]
+        parts = version.split(".")
+        if len(parts) == 2:
+            version = "v{}r{}".format(*parts)
+        elif len(parts) == 3:
+            version = "v{}r{}p{}".format(*parts)
+        else:
+            pass  # we have a convention only for 2 or 3 parts versions
+
     if name:
         # standard project suffixes
         suffixes = [
@@ -381,6 +419,7 @@ def listPlatforms(name, version, allow_empty_version=False, quiet=False):
     return sorted(platforms, key=platform_sort_key, reverse=True)
 
 
+@lru_cache()
 def findDataPackage(name, version):
     """
     Find a data package in the directories specified in the 'path' variable,
@@ -395,6 +434,16 @@ def findDataPackage(name, version):
     """
     from fnmatch import fnmatch
 
+    if re.match(r"^\d[\d.]*$", version):
+        # this is a X.Y CMake style version, so we map it to vXr*
+        # - get the first 3 element max (we only support vXrYpZ in packages)
+        parts = version.split(".")[:3]
+        # - the first elaments are constrained, but the last is free
+        parts[-1] = "*"
+        # - join the parts in a string prefixing each of them with the
+        #   right positional prefix
+        version = "".join(a + b for a, b in zip("vrp", parts))
+
     suffixes = ["", "DBASE", "PARAM"]
     versions = []
     for p in [os.path.join(b, s, name) for b in path for s in suffixes]:
@@ -412,6 +461,7 @@ def findDataPackage(name, version):
     return os.path.join(p, v)
 
 
+@lru_cache()
 def parseManifest(manifest):
     """
     Extract the list of required projects and data packages from a manifest.xml
@@ -446,6 +496,28 @@ def parseManifest(manifest):
     return (used_projects, data_packages)
 
 
+def getManifests(project, version, platform, allow_empty_version=False):
+    """
+    Return an iterator over all manifest.xml files from the project to
+    the dependencies.
+    """
+    pdir = findProject(
+        project, version, platform, allow_empty_version=allow_empty_version
+    )
+    manifest = os.path.join(pdir, "manifest.xml")
+    if not os.path.exists(manifest):
+        raise MissingManifestError(manifest)
+
+    yield manifest
+
+    projects, _ = parseManifest(manifest)
+    for p, v in projects:
+        for manifest in getManifests(
+            p, v, platform, allow_empty_version=allow_empty_version
+        ):
+            yield manifest
+
+
 def getEnvXmlPath(project, version, platform, allow_empty_version=False):
     """
     Return the list of directories to be added to the Env XML search path for
@@ -455,20 +527,16 @@ def getEnvXmlPath(project, version, platform, allow_empty_version=False):
         project, version, platform, allow_empty_version=allow_empty_version
     )
     search_path = [pdir]
-    # manifests to parse
-    manifests = [os.path.join(pdir, "manifest.xml")]
-    while manifests:
-        manifest = manifests.pop(0)
-        if not os.path.exists(manifest):
-            raise MissingManifestError(manifest)
+    for manifest in getManifests(
+        project, version, platform, allow_empty_version=allow_empty_version
+    ):
         projects, packages = parseManifest(manifest)
         # add the data package directories
-        search_path.extend([findDataPackage(p, v) for p, v in packages])
-        # add the project directories ...
-        pdirs = [findProject(p, v, platform) for p, v in projects]
-        search_path.extend(pdirs)
-        # ... and their manifests to the list of manifests to parse
-        manifests.extend([os.path.join(pdir, "manifest.xml") for pdir in pdirs])
+        search_path.extend(findDataPackage(p, v) for p, v in packages)
+        # add the project directories
+        search_path.extend(
+            findProject(p, v, platform, allow_empty_version=False) for p, v in projects
+        )
 
     def _unique(iterable):
         returned = set()
@@ -480,6 +548,31 @@ def getEnvXmlPath(project, version, platform, allow_empty_version=False):
     return list(_unique(search_path))
 
 
+def getProjectsDirs(project, version, platform, allow_empty_version=False):
+    """
+    Return a dictionary mapping project name to its root directory.
+    """
+    return {
+        getProjectNameVersion(manifest)[0]: os.path.dirname(manifest)
+        for manifest in getManifests(
+            project, version, platform, allow_empty_version=allow_empty_version
+        )
+    }
+
+
+def getPackagesDirs(project, version, platform, allow_empty_version=False):
+    """
+    Return a dictionary mapping data package name to its root directory.
+    """
+    out = {}
+    for manifest in getManifests(
+        project, version, platform, allow_empty_version=allow_empty_version
+    ):
+        _, packages = parseManifest(manifest)
+        out.update((p, findDataPackage(p, v)) for p, v in packages)
+    return out
+
+
 def findLCG(version, platform):
     """
     Return the path to the requested LCG version, found in the search path.
@@ -679,7 +772,7 @@ def walkProjectDeps(project, version, platform="any", ignore_missing=False):
     if project.lower() in ("lcg", "heptools", "lcgcmt"):
         return
     try:
-        root = findProject(project, version, platform)
+        root = findProject(project, version, platform, allow_empty_version=False)
     except MissingProjectError as err:
         if ignore_missing:
             log.warning(str(err))
