@@ -999,9 +999,13 @@ class LbRun(xenv.Script):
 
         # make sure we load the data packages too
         for p, d in data_packages.items():
-            self.opts.actions.append(
-                ("loadXML", (str("{}/{}.xenv".format(d, p.replace("/", "_"))),))
-            )
+            xml_path = "{}/{}.xenv".format(d, p.replace("/", "_"))
+            if not os.path.exists(xml_path):
+                # fall back on the old conventional name
+                xml_path = xml_path[:-5] + "Environment.xml"
+            # FIXME: xenv has got problems with unicode filenames
+            self.opts.actions.append(("loadXML", (str(xml_path),)))
+
         # handle the extra data packages
         for pkg_name, pkg_vers in map(decodePkg, self.opts.use):
             xml_name = pkg_name.replace("/", "_") + ".xenv"
